﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "账户明细")]
    [Alias(Configs.TableNameAccountDetail)]
    public class AccountDetail:EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        #region 关联信息 Account,Offset Biz
        [InternalSetter]
        [DefaultEmptyGuid, Required]
        public Guid AccountKvid { get; set; }


        [ApiMember(Description = "业务Kvid")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200), Default("")]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型,指实体类型全名称")]
        [InternalSetter]
        [StringLength(200), Default("")]
        public string BizType { get; set; }

        #endregion

        #region 收付双方 Payer付款单位,贷方  Payee收款单位,借方 禁止更新
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付款单位,贷方")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayerName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerAccountKvid { get; set; }

        [ApiMember(Description = "付款账户名称,如果是Bank类的，指银行")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerAccountName { get; set; }

        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayerAccountSerialNumber { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收款单位,借方")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayeeName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeAccountKvid { get; set; }

        [ApiMember(Description = "收款账户名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeAccountName { get; set; }

        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayeeAccountSerialNumber { get; set; }
        #endregion

        #region 金额相关
        /// <summary>
        /// 发生金额
        /// </summary>
        [ApiMember(Description = "发生金额,正数为入（借），负数为出（贷）正常全为正数，如果为负值，应该对调借贷双方")]
        [IgnoreUpdate]
        [DecimalLength(15, 2), Required]
        public decimal AmountPayment { get; set; }

        /// <summary>
        /// 帐户余额
        /// </summary>
        [ApiMember(Description = "帐户余额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 

        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }

        #endregion
    }
}
