﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "发票")]
    [Alias(Configs.TableNameInvoice)]
    public class Invoice : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset, IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        #region 关联信息 Owner,Offset Biz,全内部设置
        /// <summary>
        /// 一般不使用此字段，预留
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [InternalSetter]
        [StringLength(500)]
        public string OwnerName { get; set; }
        /// <summary>
        /// 冲帐关联
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }

        [ApiMember(Description = "收付款关联,针对Payment和Invoice多对多的关系 使用中间表Apply(开票申请)关联起来")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid ApplyKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }
        #endregion

        #region 收付双方 Payer付款单位,贷方  Payee收款单位,借方 全禁止更新
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付款单位,贷方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }

        [ApiMember(Description = "付方税号,收票方税号")]
        [IgnoreUpdate]
        [StringLength(100)]
        public string PayerTaxNumber { get; set; }

        [ApiMember(Description = "公司地址")]
        [StringLength(200), Default("")]
        public string PayerCompanyAddress { get; set; }

        [ApiMember(Description = "电话号码")]
        [StringLength(50), Default("")]
        public string PayerPhone { get; set; }

        [ApiMember(Description = "付款账户名称,指开户银行")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerRegisteredBank { get; set; }

        [ApiMember(Description = "付款账户卡号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayerBankAccount { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收款单位,借方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }

        [ApiMember(Description = "收方税号,开票方税号")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeTaxNumber { get; set; }

        [ApiMember(Description = "公司地址")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeCompanyAddress { get; set; }

        [ApiMember(Description = "电话号码")]
        [IgnoreUpdate]
        [StringLength(50), Default("")]
        public string PayeePhone { get; set; }

        [ApiMember(Description = "收款账户名称,指开户银行")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeRegisteredBank { get; set; }

        [ApiMember(Description = "收款账户卡号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayeeBankAccount { get; set; }
        #endregion

        #region 发票相关
        [ApiMember(Description = "发票代码")]
        [InternalSetter]
        [StringLength(100), Default("")]
        public string SerialCode { get; set; }

        [ApiMember(Description = "发票号码，唯一性，拆分的发票和原编号有关联性")]
        [InternalSetter, Unique]
        [StringLength(100), Default("")]
        public string SerialNumber { get; set; }

        [ApiMember(Description = "发票类型，普通，专用增值等")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public string Type { get; set; }


        [ApiMember(Description = "分类，借票,到账等")]
        [StringLength(50), Default("")]
        public string Category { get; set; }
        #endregion

        #region 金额和税
        [ApiMember(Description = "发票税率")]
        [IgnoreUpdate]
        [DecimalLength(5, 4), Default(0)]
        public decimal TaxRate { get; set; }

        [ApiMember(Description = "收款金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPayment { get; set; }

        [ApiMember(Description = "含税金额，合同或订单中使用的金额")]
        [IgnoreUpdate]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }


        [ApiMember(Description = "未税金额，Amount=AmountUntaxed+AmountTax")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountUntaxed { get; set; }

        [ApiMember(Description = "税额,和计算结果Amount-Amount/（1+TaxRate）不能相差超过一个固定值")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountTax { get; set; }
        #endregion

        #region 拆分属性
        //ParentKvid为 Empty为主发票，否则为拆分发票，不作统计用，只支持原发票拆分，拆分后不再支持再拆分
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }

        //如果被拆分后，这个金额应该是拆分项目Amount金额的统计
        [ApiMember(Description = "关联金额，被业务关联掉的金额，正常应该和Amount相同")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountUsed { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 全禁止更新

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [ApiMember(Description = "支付日期")]
        [InternalSetter]
        public DateTime? PayedTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经 办 人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }


        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }

        #endregion

        [Ignore]
        public List<InvoiceDetail> Details { get; set; }
    }
}
