﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class PaymentExtension
    {
        /// <summary>
        /// complate!
        /// 重新计算Payment的 AmountInvoice
        /// 只针对Bank等根级Payment
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Payment RecalculateAmountInvoice(this Payment payment, IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("此到账已作废 无法重新计算开票金额");
            (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Split).ThrowIfTrue("不支持的类型");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var amountInvoiced = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountInvoice), p => p.Type == PaymentType.Split && p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
            var invoiceTime = conn.Scalar<Payment, DateTime?>(o => Sql.Max(o.InvoiceTime), p => p.Type == PaymentType.Split && p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);

            Payment parentPayment = null;
            if (payment.ParentKvid != Guid.Empty) parentPayment = conn.SingleById<Payment>(payment.ParentKvid);
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                payment.AmountInvoice = amountInvoiced;
                payment.AddOnlyProperties(o => o.AmountInvoice);
                if (invoiceTime != null)
                {
                    payment.InvoiceTime = invoiceTime;
                    payment.AddOnlyProperties(o => o.InvoiceTime);
                }
                conn.UpdateOnly(payment);
                if (parentPayment != null) parentPayment.RecalculateAmountInvoice(conn);
                trans?.Commit();
                return payment;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
}
