﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    #region CRQ
    /// <summary>
    /// 开户功能
    /// </summary>
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class AccountCreate : RestfulCreate<Account>
    {
        private IDbTransaction _trans;
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<Account> rtns)
        {
            #region 数据有效性验证
            //if (Item.OwnerKvid == Guid.Empty) throw new Exception("未指定账户所有人");
            if (Item.OwnerName.IsNullOrEmpty()) throw new Exception("未指定账户所有人名称");
            if (Item.Currency == CurrencyUnit.Unsupported) throw new Exception("不支持的货币类型！");
            if (Item.Type == AccountType.Unsupported) throw new Exception("不支持的账户类型！");
            //一个客户或用户只能拥有一个存款账户
            var exist = dbConnection.Exists<Account>(o => o.OwnerName == Item.OwnerName && o.Type == Item.Type);
            exist.ThrowIfTrue($"客户{Item.OwnerName}已经存在当前账户类型：{Item.Type.GetDescription()}");
            #endregion
            _trans = dbConnection.OpenTransaction();
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<Account> rtns)
        {
            try
            {

                //新建开户记录
                var accountDetail = new AccountDetail();
                accountDetail.AccountKvid = Item.Kvid;
                accountDetail.BizKvid = Item.Kvid;
                accountDetail.BizType = typeof(Account).FullName;
                accountDetail.BizId = Item.SerialNumber;

                accountDetail.PayerKvid = Item.Kvid;
                accountDetail.PayerName = Item.Name;
                accountDetail.PayerAccountKvid = Item.Kvid;
                accountDetail.PayerAccountName = Item.Name;
                accountDetail.PayerAccountSerialNumber = Item.SerialNumber;

                accountDetail.PayeeKvid = Item.Kvid;
                accountDetail.PayeeName = Item.Name;
                accountDetail.PayeeAccountKvid = Item.Kvid;
                accountDetail.PayeeAccountName = Item.Name;
                accountDetail.PayeeAccountSerialNumber = Item.SerialNumber;

                accountDetail.AmountPayment = 0;
                accountDetail.Amount = 0;

                accountDetail.Summary = "开户";
                accountDetail.Remark = "开户";
                dbConnection.Insert(accountDetail);

                _trans?.Commit();
                return base.OnPostRestfulCreate(req, res, dbConnection, rtns);
            }
            catch (Exception ex)
            {
                _trans?.Rollback();
                throw ex;
            }
            finally { _trans?.Dispose(); }
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class AccountQuery : RestfulQuery<Account>
    {
    }
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class AccountRead : RestfulRead<Account>
    {
    }

    [Api(Description = "账户查询")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class AccountQueryEx : RestfulExecution<Account>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Account>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Account>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            var ownerKvids = KiviiContext.CurrentMember.DepartmentKvids.ToList();
            ownerKvids.Add(KiviiContext.CurrentMember.OrganizationKvid);
            ownerKvids.Add(KiviiContext.CurrentMember.Kvid);
            sqlExpress.Where(o => o.Type != AccountType.Balance && Sql.In(o.OwnerKvid, ownerKvids));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    #endregion

    /// <summary>
    /// 获取账户类型
    /// </summary>
    [Api(Description = "获取账户类型")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class AccountTypeGet : RestfulExecution<AccountType>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            List<KeyValuePair<string, AccountType>> results = Extension.GetAllValuesAndDescriptions<AccountType>();

            var rtns = new RestfulQueryResponse<KeyValuePair<string, AccountType>>();
            rtns.Results = results;
            rtns.Total = results.Count();
            return rtns;
        }
    }
}
