﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceDetailCreate : RestfulCreate<InvoiceDetail>
    {
        private Invoice _invoice;
        private IDbTransaction _trans;
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<InvoiceDetail> rtns)
        {
            #region 数据有效性验证
            if (Item.InvoiceKvid == Guid.Empty) throw new Exception("");
            if (Item.Amount < 0 || Item.AmountTax < 0 || Item.AmountUntaxed < 0) throw new Exception("");
            _invoice = dbConnection.Select<Invoice>(o => o.Kvid == Item.InvoiceKvid).FirstOrDefault();
            if (_invoice == null) throw new Exception("无此发票");
            Item.TaxRate = _invoice.TaxRate;
            Item.AddOnlyProperties(o => o.TaxRate);
            #endregion
            if (dbConnection is LinqConnection && (dbConnection as LinqConnection).DbTransaction == null)
            {
                _trans = dbConnection.OpenTransaction();
            }
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<InvoiceDetail> rtns)
        {
            try
            {

                _invoice.RemoveAllOnlyProperties();
                _invoice.Amount = dbConnection.Scalar<InvoiceDetail, decimal>(o => Sql.Sum(o.Amount), o => o.InvoiceKvid == _invoice.Kvid);
                _invoice.AddOnlyProperties(o => o.Amount);
                dbConnection.UpdateOnly(_invoice);
                _trans?.Commit();
                return base.OnPostRestfulCreate(req, res, dbConnection, rtns);
            }
            catch (Exception ex)
            {
                _trans?.Rollback();
                throw ex;
            }
            finally { _trans?.Dispose(); }
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceDetailRead : RestfulRead<InvoiceDetail>
    { }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceDetailUpdate : RestfulUpdate<InvoiceDetail>
    { }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceDetailDelete : RestfulDelete<InvoiceDetail>
    { }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceDetailQuery : RestfulQuery<InvoiceDetail>
    { }
}
