﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceTitleCreate : RestfulExecution<InvoiceTitle>
    {
        public InvoiceTitle Item { get; set; }
        public List<InvoiceTitle> Items { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("缺少发票信息！");
            if (Items.IsNullOrEmpty()) Items = new List<InvoiceTitle>();
            if (Item != null) Items.Add(Item);
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceTitle>();

            var rtns = new RestfulCreateResponse<InvoiceTitle>();
            rtns.Results = new List<InvoiceTitle>();
            foreach (var item in Items)
            {
                if (conn.Exists<InvoiceTitle>(o => o.CompanyName == item.CompanyName)) continue;
                conn.Insert(item);
                rtns.Results.Add(item);
            }
            return rtns;
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceTitleUpdate : RestfulUpdate<InvoiceTitle>
    {
        public override bool OnPreRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<InvoiceTitle> rtns)
        {
            if (!Item.CompanyName.IsNullOrEmpty())
            {
                Item.CompanyName = Item.CompanyName.Replace("(", "（");
                Item.CompanyName = Item.CompanyName.Replace(")", "）");
                Item.CompanyName = Item.CompanyName.Replace(" ", "");
                Item.AddOnlyProperties(o => o.CompanyName);
            }
            return base.OnPreRestfulUpdate(req, res, dbConnection, rtns);
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceTitleRead : RestfulRead<InvoiceTitle>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceTitleQuery : RestfulQuery<InvoiceTitle>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class InvoiceTitleDelete : RestfulDelete<InvoiceTitle>
    {
    }
}
